<?php
//作成者：https://note.com/tekito_note
//もし不具合があった場合、ご連絡ください。

//設定
$uploadDir = 'images/'; // 画像アップロードディレクトリ
$miniDir = 'images/mini/';//サムネイルディレクトリ
$thumbnailWidth = 250;//サムネイルのサイズ
$fontFile = 'Mplus1-Regular.ttf'; //使用するフォントファイル
$resizeOption = '1'; // 画像サイズ制限オプション（1 = 有効, 0 = 無効）
$maxDimension = 1000; // 最大サイズ
$defaultWatermark = 'watermark.png'; //デフォルトの透かし画像
$watermarkScale = 0.2;//透かし画像の倍率(対横幅)
$convertToWebpOption = '0'; // WebP変換オプション（1 = 有効, 0 = 無効）

$allowedExtensions = ['jpg', 'jpeg', 'png', 'gif', 'webp'];
$fileNames = [];
$tempFiles = [];

// ディレクトリがない場合作成
if (!file_exists($uploadDir)) {
    if (!mkdir($uploadDir, 0755, true)) {
        throw new Exception("画像アップロードディレクトリの作成に失敗しました");
    }
    chmod($uploadDir, 0755);
}

if (!file_exists($miniDir)) {
    if (!mkdir($miniDir, 0755, true)) {
        throw new Exception("サムネイルディレクトリの作成に失敗しました");
    }
    chmod($miniDir, 0755);
}

// 設定値を取得するためのエンドポイント
if (isset($_GET['getConfig'])) {
    header('Content-Type: application/json');
    echo json_encode([
        'convertToWebpOption' => $convertToWebpOption,
        'resizeOption' => $resizeOption,
        'maxDimension' => $maxDimension
    ]);
    exit;
}

if ($_SERVER['REQUEST_METHOD'] === 'POST') {
    try {
        if (isset($_FILES['files']) && count($_FILES['files']['name']) > 0) {
            foreach ($_FILES['files']['tmp_name'] as $index => $fileTmpPath) {
                try {
                    // メモリ使用量をチェック
                    $memoryLimit = ini_get('memory_limit');
                    $memoryLimit = preg_replace('/[^0-9]/', '', $memoryLimit) * 1024 * 1024;
                    if (memory_get_usage() > ($memoryLimit * 0.8)) {
                        throw new Exception("メモリ不足のため処理を中断しました");
                    }

                    if ($_FILES['files']['error'][$index] === UPLOAD_ERR_OK) {
                        $fileName = $_FILES['files']['name'][$index];
                        
                        $fileName = mb_convert_encoding($fileName, 'UTF-8', 'AUTO');
                    
                        $safeFileName = preg_replace('/[^\p{L}\p{N}\s\-\_\.]/u', '', $fileName);
                        
                        $safeFileName = str_replace(' ', '_', $safeFileName);
                        
                        $fileExtension = strtolower(pathinfo($safeFileName, PATHINFO_EXTENSION));

                        if (!in_array($fileExtension, $allowedExtensions)) {
                            continue;
                        }

                        $newFileName = $safeFileName;
                        $dest_path = $uploadDir . $newFileName;

                        $counter = 1;
                        while (file_exists($dest_path)) {
                            $pathInfo = pathinfo($safeFileName);
                            $newFileName = $pathInfo['filename'] . '_' . $counter . '.' . $pathInfo['extension'];
                            $dest_path = $uploadDir . $newFileName;
                            $counter++;
                        }

                        if (move_uploaded_file($fileTmpPath, $dest_path)) {
                            $originalImagePath = $uploadDir . 'temp_original.' . $fileExtension;
                            copy($dest_path, $originalImagePath);
                        
                            removeICCProfile($dest_path);
                        
                            if ($resizeOption === '1') {
                                resizeImage($dest_path, $maxDimension);
                            }
                        
                            // メイン画像の加工（順序を整理）
                            $needsProcessing = false;

                            // テキスト挿入の処理
                            if (isset($_POST['insertText']) && $_POST['insertText'] === 'yes') {
                                $customText = $_POST['customText'];
                                addTextAreaToImage($dest_path, $customText);
                                $needsProcessing = true;
                            }
                            
                            // 透かし画像の処理
                            if (isset($_POST['applyWatermark']) && $_POST['applyWatermark'] === 'yes') {
                                $position = $_POST['watermarkPosition'] ?? 'center';
                                $watermarkPath = $_POST['watermarkFile'] ?? $defaultWatermark;
                                applyWatermark($dest_path, $watermarkPath, $position);
                                $needsProcessing = true;
                            }
                        
                            // サムネイル作成の処理
                            if (isset($_POST['createThumbnail']) && $_POST['createThumbnail'] === 'yes') {
                                if (isset($_POST['applyThumbnail']) && $_POST['applyThumbnail'] === '1' && $needsProcessing) {
                                    $thumbnailPath = createThumbnail($dest_path, $miniDir, $fileExtension);
                                } else {
                                    $thumbnailPath = createThumbnail($originalImagePath, $miniDir, $fileExtension);
                                }
                
                                $thumbnailNewPath = $miniDir . pathinfo($newFileName, PATHINFO_FILENAME) . '.' . $fileExtension;
                                rename($thumbnailPath, $thumbnailNewPath);
                
                                if ($convertToWebpOption === '1' && $fileExtension !== 'webp') {
                                    convertThumbnailToWebp($thumbnailNewPath, $miniDir);
                                }
                            }

                            // WebP変換の処理（サムネイル作成とは独立して実行）
                            if ($convertToWebpOption === '1' && $fileExtension !== 'webp') {
                                $dest_path = convertToWebp($dest_path, $uploadDir, $fileExtension);
                            }
                        
                            unlink($originalImagePath);
                            
                            $fileNames[] = basename($dest_path);
                        }
                    }
                } catch (Exception $e) {
                    error_log("Error processing file: " . $e->getMessage());
                    http_response_code(500);
                    echo "ERROR:" . $e->getMessage();
                    return;
                }
            }
            echo implode(',', $fileNames);
        }
    } catch (Exception $e) {
        error_log("Error in main process: " . $e->getMessage());
        http_response_code(500);
        echo "ERROR:" . $e->getMessage();
    }
}

// シャットダウン時の一時ファイル削除処理
register_shutdown_function(function() use (&$tempFiles) {
    foreach ($tempFiles as $index => $tempFile) {
        cleanupTempFile($tempFile);
        unset($tempFiles[$index]);
    }
});

// ICCプロファイル除去処理
function removeICCProfile($filePath) {
    $extension = strtolower(pathinfo($filePath, PATHINFO_EXTENSION));
    $image = null;
    
    switch ($extension) {
        case 'jpg': case 'jpeg':
            $image = @imagecreatefromjpeg($filePath);
            break;
        case 'png':
            $image = @imagecreatefrompng($filePath);
            break;
        case 'gif':
            $image = @imagecreatefromgif($filePath);
            break;
        case 'webp':
            $image = @imagecreatefromwebp($filePath);
            break;
    }

    if ($image !== false) {
        imagealphablending($image, false);
        imagesavealpha($image, true);

        switch ($extension) {
            case 'jpg': case 'jpeg':
                imagejpeg($image, $filePath);
                break;
            case 'png':
                imagepng($image, $filePath);
                break;
            case 'gif':
                imagegif($image, $filePath);
                break;
            case 'webp':
                imagewebp($image, $filePath);
                break;
        }
        
        imagedestroy($image);
    }
}

//リサイズ処理
function resizeImage($filePath, $maxDimension) {
    list($width, $height, $type) = getimagesize($filePath);
    if ($width <= $maxDimension && $height <= $maxDimension) {
        return;
    }

    $aspectRatio = $width / $height;
    if ($width > $height) {
        $newWidth = (int)$maxDimension;
        $newHeight = (int)($maxDimension / $aspectRatio);
    } else {
        $newHeight = (int)$maxDimension;
        $newWidth = (int)($maxDimension * $aspectRatio);
    }

    switch ($type) {
        case IMAGETYPE_JPEG:
            $source = imagecreatefromjpeg($filePath);
            break;
        case IMAGETYPE_PNG:
            $source = imagecreatefrompng($filePath);
            break;
        case IMAGETYPE_GIF:
            $source = imagecreatefromgif($filePath);
            break;
        case IMAGETYPE_WEBP:
            $source = imagecreatefromwebp($filePath);
            break;
        default:
            return;
    }

    $resizedImage = imagecreatetruecolor($newWidth, $newHeight);
    imagecopyresampled($resizedImage, $source, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);

    switch ($type) {
        case IMAGETYPE_JPEG:
            imagejpeg($resizedImage, $filePath);
            break;
        case IMAGETYPE_PNG:
            imagepng($resizedImage, $filePath);
            break;
        case IMAGETYPE_GIF:
            imagegif($resizedImage, $filePath);
            break;
        case IMAGETYPE_WEBP:
            imagewebp($resizedImage, $filePath);
            break;
    }

    imagedestroy($source);
    imagedestroy($resizedImage);
}
// サムネイル作成
function createThumbnail($filePath, $miniDir, $fileExtension) {
    global $thumbnailWidth; 
    
    removeICCProfile($filePath);
    
    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            $sourceImage = imagecreatefromjpeg($filePath);
            break;
        case 'png':
            $sourceImage = imagecreatefrompng($filePath);
            break;
        case 'gif':
            $sourceImage = imagecreatefromgif($filePath);
            break;
        case 'webp':
            $sourceImage = imagecreatefromwebp($filePath);
            break;
    }

    $sourceWidth = imagesx($sourceImage);
    $sourceHeight = imagesy($sourceImage);
    $thumbnailHeight = (int) ($sourceHeight * $thumbnailWidth / $sourceWidth);
    $thumbnailImage = imagecreatetruecolor($thumbnailWidth, $thumbnailHeight);

    $white = imagecolorallocate($thumbnailImage, 255, 255, 255);
    imagefill($thumbnailImage, 0, 0, $white);
    imagecopyresampled($thumbnailImage, $sourceImage, 0, 0, 0, 0, $thumbnailWidth, $thumbnailHeight, $sourceWidth, $sourceHeight);

    $thumbnailPath = $miniDir . pathinfo($filePath, PATHINFO_FILENAME) . '.' . pathinfo($filePath, PATHINFO_EXTENSION);
    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            imagejpeg($thumbnailImage, $thumbnailPath);
            break;
        case 'png':
            imagepng($thumbnailImage, $thumbnailPath);
            break;
        case 'gif':
            imagegif($thumbnailImage, $thumbnailPath);
            break;
        case 'webp':
            imagewebp($thumbnailImage, $thumbnailPath);
            break;
    }

    imagedestroy($sourceImage);
    imagedestroy($thumbnailImage);

    return $thumbnailPath;
}


// 画像にテキスト挿入
function addTextAreaToImage($filePath, $text) {
    global $fontFile;
    
    $text = str_replace(
        ['<', '>', '&', '"', "'"],
        ['[LESS]', '[GREATER]', '[AMP]', '[QUOTE]', '[SINGLE]'],
        $text
    );
    
    $text = htmlspecialchars($text, ENT_QUOTES, 'UTF-8');
    
    $text = str_replace(
        ['[LESS]', '[GREATER]', '[AMP]', '[QUOTE]', '[SINGLE]'],
        ['<', '>', '&', '"', "'"],
        $text
    );
    
    $text = preg_replace('/[[:cntrl:]]/', '', $text);

    $fileExtension = pathinfo($filePath, PATHINFO_EXTENSION);
    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            $image = imagecreatefromjpeg($filePath);
            break;
        case 'png':
            $image = imagecreatefrompng($filePath);
            break;
        case 'gif':
            $image = imagecreatefromgif($filePath);
            break;
        case 'webp':
            $image = imagecreatefromwebp($filePath);
            break;
    }

    $imageWidth = imagesx($image);
    $imageHeight = imagesy($image);
    $fontSize = $imageHeight * 0.02;
    $alpha = 0;
    $textColor = imagecolorallocatealpha($image, 0, 0, 0, $alpha);
    $outlineColor = imagecolorallocatealpha($image, 255, 255, 255, $alpha);

    $bbox = imagettfbbox($fontSize, 0, $fontFile, $text);
    $textWidth = $bbox[2] - $bbox[0];
    $xPosition = $imageWidth - $textWidth - 20;
    $yPosition = $imageHeight - 20;
    $outlineOffset = 1;

    $text = mb_convert_encoding($text, 'UTF-8', 'auto'); //文字化けする場合ここを変更
    imagettftext($image, $fontSize, 0, $xPosition - $outlineOffset, $yPosition - $outlineOffset, $outlineColor, $fontFile, $text);
    imagettftext($image, $fontSize, 0, $xPosition + $outlineOffset, $yPosition - $outlineOffset, $outlineColor, $fontFile, $text);
    imagettftext($image, $fontSize, 0, $xPosition - $outlineOffset, $yPosition + $outlineOffset, $outlineColor, $fontFile, $text);
    imagettftext($image, $fontSize, 0, $xPosition + $outlineOffset, $yPosition + $outlineOffset, $outlineColor, $fontFile, $text);
    imagettftext($image, $fontSize, 0, $xPosition, $yPosition, $textColor, $fontFile, $text);

    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            imagejpeg($image, $filePath);
            break;
        case 'png':
            imagepng($image, $filePath);
            break;
        case 'gif':
            imagegif($image, $filePath);
            break;
        case 'webp':
            imagewebp($image, $filePath);
            break;
    }

    imagedestroy($image);
}

// WebP変換
function convertToWebp($filePath, $uploadDir, $fileExtension) {
    $webpPath = $uploadDir . pathinfo($filePath, PATHINFO_FILENAME) . '.webp';
    $image = null;

    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            $image = imagecreatefromjpeg($filePath);
            break;
        case 'png':
            $image = imagecreatefrompng($filePath);
            break;
        case 'gif':
            $image = imagecreatefromgif($filePath);
            break;
    }

    if ($image) {
        imagewebp($image, $webpPath);
        imagedestroy($image);
        unlink($filePath);
    }

    return $webpPath;
}

function convertThumbnailToWebp($thumbnailPath, $miniDir) {
    $fileExtension = strtolower(pathinfo($thumbnailPath, PATHINFO_EXTENSION));
    $webpThumbnailPath = $miniDir . pathinfo($thumbnailPath, PATHINFO_FILENAME) . '.webp';

    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            $image = imagecreatefromjpeg($thumbnailPath);
            break;
        case 'png':
            $image = imagecreatefrompng($thumbnailPath);
            break;
        case 'gif':
            $image = imagecreatefromgif($thumbnailPath);
            break;
        case 'webp':
            $image = imagecreatefromwebp($thumbnailPath);
            break;
        default:
            $image = false;
            break;
    }

    if ($image !== false) {
        imagewebp($image, $webpThumbnailPath);
        imagedestroy($image);
        unlink($thumbnailPath);
    } else {
        echo "サムネイルの WebP 変換に失敗しました。対応していない画像形式か、ファイルが破損している可能性があります。";
    }
}

// 透かし画像
function applyWatermark($filePath, $watermarkPath, $position) {
    global $watermarkScale;

    if (!$watermarkPath || !file_exists($watermarkPath)) {
        error_log("警告: 透かし画像 '{$watermarkPath}' が見つかりません。処理をスキップします。");
        return;
    }
    
    $watermarkImage = @imagecreatefrompng($watermarkPath);
    if (!$watermarkImage) {
        error_log("警告: 透かし画像 '{$watermarkPath}' を読み込めません。処理をスキップします。");
        return;
    }

    removeICCProfile($watermarkPath);

    $fileExtension = pathinfo($filePath, PATHINFO_EXTENSION);
    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            $image = imagecreatefromjpeg($filePath);
            break;
        case 'png':
            $image = imagecreatefrompng($filePath);
            break;
        case 'gif':
            $image = imagecreatefromgif($filePath);
            break;
        case 'webp':
            $image = imagecreatefromwebp($filePath);
            break;
        default:
            imagedestroy($watermarkImage);
            return;
    }

    list($imageWidth, $imageHeight) = getimagesize($filePath);
    $watermarkImage = imagecreatefrompng($watermarkPath);

    list($watermarkWidth, $watermarkHeight) = getimagesize($watermarkPath);

    $newWatermarkWidth = floor($imageWidth * $watermarkScale);
    $newWatermarkHeight = floor(($watermarkHeight / $watermarkWidth) * $newWatermarkWidth);

    $resizedWatermark = imagecreatetruecolor($newWatermarkWidth, $newWatermarkHeight);
    imagealphablending($resizedWatermark, false);
    imagesavealpha($resizedWatermark, true);
    imagecopyresampled($resizedWatermark, $watermarkImage, 0, 0, 0, 0, $newWatermarkWidth, $newWatermarkHeight, $watermarkWidth, $watermarkHeight);

    $watermarkWidth = $newWatermarkWidth;
    $watermarkHeight = $newWatermarkHeight;

    if ($position == 'repeat') {
        for ($x = 0; $x < $imageWidth; $x += $watermarkWidth) {
            for ($y = 0; $y < $imageHeight; $y += $watermarkHeight) {
                imagecopy($image, $resizedWatermark, $x, $y, 0, 0, $watermarkWidth, $watermarkHeight);
            }
        }
    } else {
        switch ($position) {
            case 'center':
                $x = floor(($imageWidth - $watermarkWidth) / 2);
                $y = floor(($imageHeight - $watermarkHeight) / 2);
                break;
            case 'top-left':
                $x = 0;
                $y = 0;
                break;
            case 'top-right':
                $x = floor($imageWidth - $watermarkWidth);
                $y = 0;
                break;
            case 'bottom-left':
                $x = 0;
                $y = floor($imageHeight - $watermarkHeight);
                break;
            case 'bottom-right':
                $x = floor($imageWidth - $watermarkWidth);
                $y = floor($imageHeight - $watermarkHeight);
                break;
            default:
                $x = 0;
                $y = 0;
                break;
        }

        imagealphablending($image, true);
        imagesavealpha($image, true);
        imagealphablending($resizedWatermark, true);
        imagesavealpha($resizedWatermark, true);

        imagecopy($image, $resizedWatermark, $x, $y, 0, 0, $watermarkWidth, $watermarkHeight);
    }

    switch ($fileExtension) {
        case 'jpg': case 'jpeg':
            imagejpeg($image, $filePath);
            break;
        case 'png':
            imagepng($image, $filePath);
            break;
        case 'gif':
            imagegif($image, $filePath);
            break;
        case 'webp':
            imagewebp($image, $filePath);
            break;
    }

    imagedestroy($image);
    imagedestroy($resizedWatermark);
    imagedestroy($watermarkImage);
}

function cleanupTempFile($filePath) {
    if (!empty($filePath) && file_exists($filePath)) {
        try {
            unlink($filePath);
            error_log("Temporary file removed: " . $filePath);
        } catch (Exception $e) {
            error_log("Failed to remove temporary file: " . $filePath . " - " . $e->getMessage());
        }
    }
}

function processImage($filePath) {
    if (isset($image)) {
        imagedestroy($image);
    }
    gc_collect_cycles();
}

function handleError($message, $logMessage = null) {
    error_log($logMessage ?? $message);
    http_response_code(500);
    echo "ERROR:" . $message;
    return;
}

?>
